--[[

	XPPercent: A very simple and lightweight XP percentage indicator
	 2010 by DeathMaster

]]

XPPERCENT_VERSION = 3.5;

--------------------------------------------------------------------------------------------------
-- Localizable strings
--------------------------------------------------------------------------------------------------

XPPERCENT = "XPPercent";

XPPERCENT_HELP = "help";
XPPERCENT_LOCK = "lock";
XPPERCENT_UNLOCK = "unlock";
XPPERCENT_RESET = "reset";

XPPERCENT_LOCKED = "XPPercent: Locked at its current position.";
XPPERCENT_UNLOCKED = "XPPercent: Unlocked and can be replaced.";
XPPERCENT_RESET_DONE = "XPPercent: Position reset to default position.";

XPPERCENT_HELP_TEXT0 = " ";
XPPERCENT_HELP_TEXT1 = "|cff00ff00XPPercent command help:|r";
XPPERCENT_HELP_TEXT2 = "|cff00ff00Use |r|cffffffff/xp <command>|r|cff00ff00 to perform the following commands:|r";
XPPERCENT_HELP_TEXT3 = "|cffffffff"..XPPERCENT_HELP.."|r|cff00ff00: Displays this message.|r";
XPPERCENT_HELP_TEXT4 = "|cffffffff"..XPPERCENT_LOCK.."|r|cff00ff00: Locks XPPercent so that it can't (accidentally) be replaced.|r";
XPPERCENT_HELP_TEXT5 = "|cffffffff"..XPPERCENT_UNLOCK.."|r|cff00ff00: Unlocks XPPercent so that it can be replaced.|r";
XPPERCENT_HELP_TEXT6 = "|cffffffff"..XPPERCENT_RESET.."|r|cff00ff00: Resets XPPercent to its default position.|r";

XPPERCENT_HELP_TEXT = {
	XPPERCENT_HELP_TEXT0,
	XPPERCENT_HELP_TEXT1,
	XPPERCENT_HELP_TEXT2,
	XPPERCENT_HELP_TEXT3,
	XPPERCENT_HELP_TEXT4,
	XPPERCENT_HELP_TEXT5,
	XPPERCENT_HELP_TEXT6,
};

--------------------------------------------------------------------------------------------------
-- Local variables
--------------------------------------------------------------------------------------------------

local XPPercentState;
local lBeingDragged;

--------------------------------------------------------------------------------------------------
-- Internal functions
--------------------------------------------------------------------------------------------------

function XPPercent_SetXP()
	XPPercentText:SetText("XP " .. floor((UnitXP("player") / UnitXPMax("player")) * 100) .. "%");
end

local function XPPercent_Reset()
	XPPercentFrame:ClearAllPoints();
	XPPercentFrame:SetPoint("TOP", "UIParent", "TOP", 0, 0);
end

function XPPercent_SlashCommandHandler(msg)
	if( msg ) then
		local command = string.lower(msg);
		if( command == "" or command == XPPERCENT_HELP ) then
			local index;
			local value;
			for index, value in XPPERCENT_HELP_TEXT do
				DEFAULT_CHAT_FRAME:AddMessage(value);
			end
		elseif( command == XPPERCENT_LOCK ) then
			XPPercentState.Lock = 1;
			XPPercent_OnDragStop();
			DEFAULT_CHAT_FRAME:AddMessage(XPPERCENT_LOCKED);
		elseif( command == XPPERCENT_UNLOCK ) then
			XPPercentState.Lock = nil;
			DEFAULT_CHAT_FRAME:AddMessage(XPPERCENT_UNLOCKED);
		elseif( command == XPPERCENT_RESET ) then
			XPPercent_Reset();
			XPPercent_OnDragStop();
			DEFAULT_CHAT_FRAME:AddMessage(XPPERCENT_RESET_DONE);
		end
	end
end

--------------------------------------------------------------------------------------------------
-- OnFoo functions
--------------------------------------------------------------------------------------------------

function XPPercent_OnLoad()
	this:RegisterForDrag("LeftButton");

	-- Register our slash command
	SLASH_XPPERCENT1 = "/xp";
	SlashCmdList["XPPERCENT"] = function(msg)
		XPPercent_SlashCommandHandler(msg);
	end
	
	this:RegisterEvent("VARIABLES_LOADED");
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this:RegisterEvent("PLAYER_XP_UPDATE");
	
	XPPercentFrame.TimeSinceLastUpdate = 0;

	if( DEFAULT_CHAT_FRAME ) then
		DEFAULT_CHAT_FRAME:AddMessage("XPPercent: Loaded");
	end
	UIErrorsFrame:AddMessage("XPPercent: Loaded", 2.2, 2.2, 2.2, 2.2, UIERRORS_HOLD_TIME);
end

function XPPercent_OnEvent()
	if( event == "PLAYER_ENTERING_WORLD" ) then
               local tplayer = UnitName("player") .. " of " .. GetCVar ("realmName")

                -- Configure saved variable, first.
                if (not XPPercent) then XPPercent = { } end
                if (not XPPercent[tplayer]) then
                   XPPercent[tplayer] = { }
                end

                -- Now, reference it via the convenience variable.
                XPPercentState = XPPercent[tplayer]

		XPPercent_SetXP();
	elseif( event == "PLAYER_XP_UPDATE" ) then
		XPPercent_SetXP();
	elseif( event == "VARIABLES_LOADED" ) then
		if( not XPPercent ) then
			XPPercent = { };
		end

 		-- XPPercent to myAddOns addons list
		if(myAddOnsList) then
			myAddOnsList.XPPercent = {name = 'XPPercent', description = 'XPPercent', version = XPPERCENT_VERSION, category = MYADDONS_CATEGORY_BARS, frame = 'XPPercentFrame'};
		end
	end
end

function XPPercent_OnDragStart()
	if( not XPPercentState or not XPPercentState.Lock ) then
		XPPercentFrame:StartMoving();
		lBeingDragged = 1;
	end
end

function XPPercent_OnDragStop()
	XPPercentFrame:StopMovingOrSizing();
	lBeingDragged = nil;
end
